import React, { useEffect, useRef, useState } from 'react';
import * as THREE from 'three';
import { useEventsStore } from '../../../../../store/simulation/useEventsStore';
import useModuleStore from '../../../../../store/useModuleStore';
import { TransformControls } from '@react-three/drei';
import { detectModifierKeys } from '../../../../../utils/shortcutkeys/detectModifierKeys';

function PointsCreator() {
    const { events, updatePoint, getPointByUuid } = useEventsStore();
    const { activeModule } = useModuleStore();
    const transformRef = useRef<any>(null);
    const [transformMode, setTransformMode] = useState<"translate" | "rotate" | null>(null);
    const [selectedPoint, setSelectedPoint] = useState<THREE.Mesh | null>(null);
    const sphereRefs = useRef<{ [key: string]: THREE.Mesh }>({});

    useEffect(() => {
        setTransformMode(null);
        const handleKeyDown = (e: KeyboardEvent) => {
            const keyCombination = detectModifierKeys(e);
            if (!selectedPoint) return;
            if (keyCombination === "G") {
                setTransformMode((prev) => (prev === "translate" ? null : "translate"));
            }
            if (keyCombination === "R") {
                setTransformMode((prev) => (prev === "rotate" ? null : "rotate"));
            }
        };

        window.addEventListener("keydown", handleKeyDown);
        return () => window.removeEventListener("keydown", handleKeyDown);
    }, [selectedPoint]);

    const updatePointToState = (selectedPoint: THREE.Mesh) => {
        let point = JSON.parse(JSON.stringify(getPointByUuid(selectedPoint.userData.modelUuid, selectedPoint.userData.pointUuid)));
        if (point) {
            point.position = [selectedPoint.position.x, selectedPoint.position.y, selectedPoint.position.z];
            updatePoint(selectedPoint.userData.modelUuid, selectedPoint.userData.pointUuid, point)
        }
    }

    return (
        <>
            {activeModule === 'simulation' &&
                <>
                    <group name='EventPointsGroup' >
                        {events.map((event, i) => {
                            if (event.type === 'transfer') {
                                return (
                                    <group key={i} position={new THREE.Vector3(...event.position)}>
                                        {event.points.map((point, j) => (
                                            <mesh
                                                uuid={point.uuid}
                                                ref={(el) => (sphereRefs.current[point.uuid] = el!)}
                                                onClick={(e) => {
                                                    e.stopPropagation();
                                                    setSelectedPoint(sphereRefs.current[point.uuid]);
                                                }}
                                                onPointerMissed={() => {
                                                    setSelectedPoint(null);
                                                }}
                                                key={`${i}-${j}`}
                                                position={new THREE.Vector3(...point.position)}
                                                userData={{ modelUuid: event.modelUuid, pointUuid: point.uuid }}
                                            >
                                                <sphereGeometry args={[0.1, 16, 16]} />
                                                <meshStandardMaterial color="orange" />
                                            </mesh>
                                        ))}
                                    </group>
                                );
                            } else if (event.type === 'vehicle') {
                                return (
                                    <group key={i} position={new THREE.Vector3(...event.position)}>
                                        <mesh
                                            uuid={event.point.uuid}
                                            ref={(el) => (sphereRefs.current[event.point.uuid] = el!)}
                                            onClick={(e) => {
                                                e.stopPropagation();
                                                setSelectedPoint(sphereRefs.current[event.point.uuid]);
                                            }}
                                            onPointerMissed={() => {
                                                setSelectedPoint(null);
                                            }}
                                            position={new THREE.Vector3(...event.point.position)}
                                            userData={{ modelUuid: event.modelUuid, pointUuid: event.point.uuid }}
                                        >
                                            <sphereGeometry args={[0.1, 16, 16]} />
                                            <meshStandardMaterial color="blue" />
                                        </mesh>
                                    </group>
                                );
                            } else if (event.type === 'roboticArm') {
                                return (
                                    <group key={i} position={new THREE.Vector3(...event.position)}>
                                        <mesh
                                            uuid={event.point.uuid}
                                            ref={(el) => (sphereRefs.current[event.point.uuid] = el!)}
                                            onClick={(e) => {
                                                e.stopPropagation();
                                                setSelectedPoint(sphereRefs.current[event.point.uuid]);
                                            }}
                                            onPointerMissed={() => {
                                                setSelectedPoint(null);
                                            }}
                                            position={new THREE.Vector3(...event.point.position)}
                                            userData={{ modelUuid: event.modelUuid, pointUuid: event.point.uuid }}
                                        >
                                            <sphereGeometry args={[0.1, 16, 16]} />
                                            <meshStandardMaterial color="green" />
                                        </mesh>
                                    </group>
                                );
                            } else if (event.type === 'machine') {
                                return (
                                    <group key={i} position={new THREE.Vector3(...event.position)}>
                                        <mesh
                                            uuid={event.point.uuid}
                                            ref={(el) => (sphereRefs.current[event.point.uuid] = el!)}
                                            onClick={(e) => {
                                                e.stopPropagation();
                                                setSelectedPoint(sphereRefs.current[event.point.uuid]);
                                            }}
                                            onPointerMissed={() => {
                                                setSelectedPoint(null);
                                            }}
                                            position={new THREE.Vector3(...event.point.position)}
                                            userData={{ modelUuid: event.modelUuid, pointUuid: event.point.uuid }}
                                        >
                                            <sphereGeometry args={[0.1, 16, 16]} />
                                            <meshStandardMaterial color="purple" />
                                        </mesh>
                                    </group>
                                );
                            } else {
                                return null;
                            }
                        })}
                    </group>
                    {(selectedPoint && transformMode) &&
                        <TransformControls ref={transformRef} object={selectedPoint} mode={transformMode} onMouseUp={(e) => { updatePointToState(selectedPoint) }} />
                    }
                </>
            }
        </>
    );
}

export default PointsCreator;
